@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,12 +16,13 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"sfconfig.h"
+
 #include	<stdio.h>
 #include	<string.h>
 #include	<math.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"common.h"
 
 static SF_FORMAT_INFO const simple_formats [] =
@@ -46,10 +47,26 @@ static SF_FORMAT_INFO const simple_formats [] =
 		"AU (Sun/Next 8-bit u-law)", "au"
 		},
 
+	{	SF_FORMAT_CAF | SF_FORMAT_PCM_16,
+		"CAF (Apple 16 bit PCM)", "caf"
+		},
+
+#if HAVE_EXTERNAL_LIBS
+	{	SF_FORMAT_FLAC | SF_FORMAT_PCM_16,
+		"FLAC 16 bit", "flac"
+		},
+#endif
+
 	{	SF_FORMAT_RAW | SF_FORMAT_VOX_ADPCM,
 		"OKI Dialogic VOX ADPCM", "vox"
 		},
 
+#if HAVE_EXTERNAL_LIBS
+	{	SF_FORMAT_OGG | SF_FORMAT_VORBIS,
+		"Ogg Vorbis (Xiph Foundation)", "oga"
+		},
+#endif
+
 	{	SF_FORMAT_WAV | SF_FORMAT_PCM_16,
 		"WAV (Microsoft 16 bit PCM)", "wav"
 		},
@@ -82,7 +99,7 @@ psf_get_format_simple (SF_FORMAT_INFO *data)
 {	int indx ;
 
 	if (data->format < 0 || data->format >= (SIGNED_SIZEOF (simple_formats) / SIGNED_SIZEOF (SF_FORMAT_INFO)))
-		return SFE_BAD_CONTROL_CMD ;
+		return SFE_BAD_COMMAND_PARAM ;
 
 	indx = data->format ;
 	memcpy (data, &(simple_formats [indx]), SIGNED_SIZEOF (SF_FORMAT_INFO)) ;
@@ -98,22 +115,32 @@ static SF_FORMAT_INFO const major_formats [] =
 {
 	{	SF_FORMAT_AIFF,		"AIFF (Apple/SGI)",						"aiff" 	},
 	{	SF_FORMAT_AU,		"AU (Sun/NeXT)", 						"au"	},
+	{	SF_FORMAT_AVR,		"AVR (Audio Visual Research)",	 		"avr"	},
+	{	SF_FORMAT_CAF,		"CAF (Apple Core Audio File)",	 		"caf"	},
+#if HAVE_EXTERNAL_LIBS
+	{	SF_FORMAT_FLAC,		"FLAC (FLAC Lossless Audio Codec)",	 	"flac"	},
+#endif
 	{	SF_FORMAT_HTK,		"HTK (HMM Tool Kit)",					"htk"	},
 	{	SF_FORMAT_SVX,		"IFF (Amiga IFF/SVX8/SV16)",			"iff"	},
 	{	SF_FORMAT_MAT4,		"MAT4 (GNU Octave 2.0 / Matlab 4.2)",	"mat"	},
 	{	SF_FORMAT_MAT5,		"MAT5 (GNU Octave 2.1 / Matlab 5.0)",	"mat"	},
+	{	SF_FORMAT_MPC2K,	"MPC (Akai MPC 2k)",					"mpc"	},
+#if HAVE_EXTERNAL_LIBS
+	{	SF_FORMAT_OGG,		"OGG (OGG Container format)",		 	"oga"	},
+#endif
 	{	SF_FORMAT_PAF,		"PAF (Ensoniq PARIS)", 					"paf"	},
 	{	SF_FORMAT_PVF,		"PVF (Portable Voice Format)",			"pvf"	},
 	{	SF_FORMAT_RAW,		"RAW (header-less)",				 	"raw"	},
+	{	SF_FORMAT_RF64,		"RF64 (RIFF 64)",						"rf64"	},
+	{	SF_FORMAT_SD2,		"SD2 (Sound Designer II)", 				"sd2"	},
 	{	SF_FORMAT_SDS,		"SDS (Midi Sample Dump Standard)", 		"sds"	},
-	/* Not ready for mainstream use yet.
-	{	SF_FORMAT_SD2,		"SD2 (Sound Designer II)", 			"sd2"	},
-	*/
 	{	SF_FORMAT_IRCAM,	"SF (Berkeley/IRCAM/CARL)",				"sf"	},
 	{	SF_FORMAT_VOC,		"VOC (Creative Labs)",					"voc"	},
 	{	SF_FORMAT_W64,		"W64 (SoundFoundry WAVE 64)",			"w64"	},
 	{	SF_FORMAT_WAV,		"WAV (Microsoft)",						"wav"	},
 	{	SF_FORMAT_NIST,		"WAV (NIST Sphere)",	 				"wav"	},
+	{	SF_FORMAT_WAVEX,	"WAVEX (Microsoft)",					"wav"	},
+	{	SF_FORMAT_WVE,		"WVE (Psion Series 3)",					"wve"	},
 	{	SF_FORMAT_XI,		"XI (FastTracker 2)",					"xi"	},
 
 } ; /* major_formats */
@@ -128,7 +155,7 @@ psf_get_format_major (SF_FORMAT_INFO *data)
 {	int indx ;
 
 	if (data->format < 0 || data->format >= (SIGNED_SIZEOF (major_formats) / SIGNED_SIZEOF (SF_FORMAT_INFO)))
-		return SFE_BAD_CONTROL_CMD ;
+		return SFE_BAD_COMMAND_PARAM ;
 
 	indx = data->format ;
 	memcpy (data, &(major_formats [indx]), SIGNED_SIZEOF (SF_FORMAT_INFO)) ;
@@ -169,6 +196,10 @@ static SF_FORMAT_INFO subtype_formats [] =
 
 	{	SF_FORMAT_DPCM_16,		"16 bit DPCM",			NULL 	},
 	{	SF_FORMAT_DPCM_8,		"8 bit DPCM",			NULL 	},
+
+#if HAVE_EXTERNAL_LIBS
+	{	SF_FORMAT_VORBIS,		"Vorbis",				NULL 	},
+#endif
 } ; /* subtype_formats */
 
 int
@@ -181,7 +212,7 @@ psf_get_format_subtype (SF_FORMAT_INFO *data)
 {	int indx ;
 
 	if (data->format < 0 || data->format >= (SIGNED_SIZEOF (subtype_formats) / SIGNED_SIZEOF (SF_FORMAT_INFO)))
-		return SFE_BAD_CONTROL_CMD ;
+		return SFE_BAD_COMMAND_PARAM ;
 
 	indx = data->format ;
 	memcpy (data, &(subtype_formats [indx]), sizeof (SF_FORMAT_INFO)) ;
@@ -196,8 +227,8 @@ int
 psf_get_format_info (SF_FORMAT_INFO *data)
 {	int k, format ;
 
-	if (data->format & SF_FORMAT_TYPEMASK)
-	{	format = data->format & SF_FORMAT_TYPEMASK ;
+	if (SF_CONTAINER (data->format))
+	{	format = SF_CONTAINER (data->format) ;
 
 		for (k = 0 ; k < (SIGNED_SIZEOF (major_formats) / SIGNED_SIZEOF (SF_FORMAT_INFO)) ; k++)
 		{	if (format == major_formats [k].format)
@@ -206,8 +237,8 @@ psf_get_format_info (SF_FORMAT_INFO *data)
 				} ;
 			} ;
 		}
-	else if (data->format & SF_FORMAT_SUBMASK)
-	{	format = data->format & SF_FORMAT_SUBMASK ;
+	else if (SF_CODEC (data->format))
+	{	format = SF_CODEC (data->format) ;
 
 		for (k = 0 ; k < (SIGNED_SIZEOF (subtype_formats) / SIGNED_SIZEOF (SF_FORMAT_INFO)) ; k++)
 		{	if (format == subtype_formats [k].format)
@@ -219,7 +250,7 @@ psf_get_format_info (SF_FORMAT_INFO *data)
 
 	memset (data, 0, sizeof (SF_FORMAT_INFO)) ;
 
-	return SFE_BAD_CONTROL_CMD ;
+	return SFE_BAD_COMMAND_PARAM ;
 } /* psf_get_format_info */
 
 /*==============================================================================
@@ -228,7 +259,7 @@ psf_get_format_info (SF_FORMAT_INFO *data)
 double
 psf_calc_signal_max (SF_PRIVATE *psf, int normalize)
 {	sf_count_t	position ;
-	double 		max_val = 0.0, temp, *data ;
+	double 		max_val, temp, *data ;
 	int			k, len, readcount, save_state ;
 
 	/* If the file is not seekable, there is nothing we can do. */
@@ -243,19 +274,18 @@ psf_calc_signal_max (SF_PRIVATE *psf, int normalize)
 		} ;
 
 	save_state = sf_command ((SNDFILE*) psf, SFC_GET_NORM_DOUBLE, NULL, 0) ;
-
 	sf_command ((SNDFILE*) psf, SFC_SET_NORM_DOUBLE, NULL, normalize) ;
 
 	/* Brute force. Read the whole file and find the biggest sample. */
-	position = sf_seek ((SNDFILE*) psf, 0, SEEK_CUR) ; /* Get current position in file */
-	sf_seek ((SNDFILE*) psf, 0, SEEK_SET) ;			/* Go to start of file. */
+	/* Get current position in file */
+	position = sf_seek ((SNDFILE*) psf, 0, SEEK_CUR) ;
+	/* Go to start of file. */
+	sf_seek ((SNDFILE*) psf, 0, SEEK_SET) ;
 
-	len = sizeof (psf->buffer) / sizeof (double) ;
+	data = psf->u.dbuf ;
+	len = ARRAY_LEN (psf->u.dbuf) ;
 
-	data = (double*) psf->buffer ;
-
-	readcount = len ;
-	while (readcount > 0)
+	for (readcount = 1, max_val = 0.0 ; readcount > 0 ; /* nothing */)
 	{	readcount = sf_read_double ((SNDFILE*) psf, data, len) ;
 		for (k = 0 ; k < readcount ; k++)
 		{	temp = fabs (data [k]) ;
@@ -263,8 +293,8 @@ psf_calc_signal_max (SF_PRIVATE *psf, int normalize)
 			} ;
 		} ;
 
-	sf_seek ((SNDFILE*) psf, position, SEEK_SET) ;		/* Return to original position. */
-
+	/* Return to SNDFILE to original state. */
+	sf_seek ((SNDFILE*) psf, position, SEEK_SET) ;
 	sf_command ((SNDFILE*) psf, SFC_SET_NORM_DOUBLE, NULL, save_state) ;
 
 	return	max_val ;
@@ -293,9 +323,9 @@ psf_calc_max_all_channels (SF_PRIVATE *psf, double *peaks, int normalize)
 	position = sf_seek ((SNDFILE*) psf, 0, SEEK_CUR) ; /* Get current position in file */
 	sf_seek ((SNDFILE*) psf, 0, SEEK_SET) ;			/* Go to start of file. */
 
-	len = sizeof (psf->buffer) / sizeof (double) ;
+	len = ARRAY_LEN (psf->u.dbuf) ;
 
-	data = (double*) psf->buffer ;
+	data = psf->u.dbuf ;
 
 	chan = 0 ;
 	readcount = len ;
@@ -313,12 +343,34 @@ psf_calc_max_all_channels (SF_PRIVATE *psf, double *peaks, int normalize)
 	sf_command ((SNDFILE*) psf, SFC_SET_NORM_DOUBLE, NULL, save_state) ;
 
 	return	0 ;
-} /* psf_calc_signal_max */
+} /* psf_calc_max_all_channels */
+
+int
+psf_get_signal_max (SF_PRIVATE *psf, double *peak)
+{	int k ;
+
+	if (psf->peak_info == NULL)
+		return SF_FALSE ;
+
+	peak [0] = psf->peak_info->peaks [0].value ;
+
+	for (k = 1 ; k < psf->sf.channels ; k++)
+		peak [0] = SF_MAX (peak [0], psf->peak_info->peaks [k].value) ;
+
+	return SF_TRUE ;
+} /* psf_get_signal_max */
+
+int
+psf_get_max_all_channels (SF_PRIVATE *psf, double *peaks)
+{	int k ;
+
+	if (psf->peak_info == NULL)
+		return SF_FALSE ;
+
+	for (k = 0 ; k < psf->sf.channels ; k++)
+		peaks [k] = psf->peak_info->peaks [k].value ;
+
+	return SF_TRUE ;
+} /* psf_get_max_all_channels */
+
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 0aae0d9d-ab2b-4d70-ade3-47a534666f8e
-*/
