@@ -336,7 +336,7 @@ psf_asciiheader_printf (SF_PRIVATE *psf, const char *format, ...)
 	maxlen	= sizeof (psf->header) - maxlen ;
 
 	va_start (argptr, format) ;
-	LSF_VSNPRINTF (start, maxlen, format, argptr) ;
+	vsnprintf (start, maxlen, format, argptr) ;
 	va_end (argptr) ;
 
 	/* Make sure the string is properly terminated. */
@@ -1132,7 +1132,7 @@ psf_hexdump (const void *ptr, int len)
 		printf ("%08X: ", k) ;
 		for (m = 0 ; m < 16 && k + m < len ; m++)
 		{	printf (m == 8 ? " %02X " : "%02X ", data [k + m] & 0xFF) ;
-			ascii [m] = isprint (data [k + m]) ? data [k + m] : '.' ;
+			ascii [m] = psf_isprint (data [k + m]) ? data [k + m] : '.' ;
 			} ;
 
 		if (m <= 8) printf (" ") ;
@@ -1207,7 +1207,7 @@ psf_sanitize_string (char * cptr, int len)
 	do
 	{
 		len -- ;
-		cptr [len] = isprint (cptr [len]) ? cptr [len] : '.' ;
+		cptr [len] = psf_isprint (cptr [len]) ? cptr [len] : '.' ;
 	}
 	while (len > 0) ;
 } /* psf_sanitize_string */
@@ -1231,11 +1231,11 @@ psf_get_date_str (char *str, int maxlen)
 #endif
 
 	if (tmptr)
-		LSF_SNPRINTF (str, maxlen, "%4d-%02d-%02d %02d:%02d:%02d UTC",
+		snprintf (str, maxlen, "%4d-%02d-%02d %02d:%02d:%02d UTC",
 			1900 + timedata.tm_year, timedata.tm_mon, timedata.tm_mday,
 			timedata.tm_hour, timedata.tm_min, timedata.tm_sec) ;
 	else
-		LSF_SNPRINTF (str, maxlen, "Unknown date") ;
+		snprintf (str, maxlen, "Unknown date") ;
 
 	return ;
 } /* psf_get_date_str */
@@ -1314,6 +1314,21 @@ psf_rand_int32 (void)
 	return value ;
 } /* psf_rand_int32 */
 
+void
+append_snprintf (char * dest, size_t maxlen, const char * fmt, ...)
+{	size_t len = strlen (dest) ;
+
+	if (len < maxlen)
+	{	va_list ap ;
+
+		va_start (ap, fmt) ;
+		vsnprintf (dest + len, maxlen - len, fmt, ap) ;
+		va_end (ap) ;
+		} ;
+
+	return ;
+} /* append_snprintf */
+
 /*==============================================================================
 */
 
