@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -26,14 +26,15 @@
 ** Lidstrom and is copyright 1993 by NuEdge Development".
 */
 
+#include	"sfconfig.h"
+
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
+#include	<math.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
-#include	"float_cast.h"
 #include	"common.h"
 
 typedef struct
@@ -54,10 +55,10 @@ static sf_count_t dwvw_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t dwvw_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t dwvw_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t dwvw_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t dwvw_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t dwvw_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t dwvw_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t dwvw_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t dwvw_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t dwvw_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t dwvw_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
 static sf_count_t	dwvw_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset) ;
 static int	dwvw_close	(SF_PRIVATE *psf) ;
@@ -65,7 +66,7 @@ static int	dwvw_close	(SF_PRIVATE *psf) ;
 static int	dwvw_decode_data (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, int *ptr, int len) ;
 static int	dwvw_decode_load_bits (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, int bit_count) ;
 
-static int	dwvw_encode_data (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, int *ptr, int len) ;
+static int	dwvw_encode_data (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, const int *ptr, int len) ;
 static void dwvw_encode_store_bits (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, int data, int new_bits) ;
 static void dwvw_read_reset (DWVW_PRIVATE *pdwvw) ;
 
@@ -77,16 +78,21 @@ int
 dwvw_init (SF_PRIVATE *psf, int bitwidth)
 {	DWVW_PRIVATE	*pdwvw ;
 
+	if (psf->codec_data != NULL)
+	{	psf_log_printf (psf, "*** psf->codec_data is not NULL.\n") ;
+		return SFE_INTERNAL ;
+		} ;
+
 	if (bitwidth > 24)
 		return SFE_DWVW_BAD_BITWIDTH ;
 
-	if (psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_RDWR)
 		return SFE_BAD_MODE_RW ;
 
 	if ((pdwvw = calloc (1, sizeof (DWVW_PRIVATE))) == NULL)
 		return SFE_MALLOC_FAILED ;
 
-	psf->fdata = (void*) pdwvw ;
+	psf->codec_data = (void*) pdwvw ;
 
 	pdwvw->bit_width 	= bitwidth ;
 	pdwvw->dwm_maxsize	= bitwidth / 2 ;
@@ -95,27 +101,27 @@ dwvw_init (SF_PRIVATE *psf, int bitwidth)
 
 	dwvw_read_reset (pdwvw) ;
 
-	if (psf->mode == SFM_READ)
+	if (psf->file.mode == SFM_READ)
 	{	psf->read_short		= dwvw_read_s ;
 		psf->read_int		= dwvw_read_i ;
 		psf->read_float		= dwvw_read_f ;
 		psf->read_double	= dwvw_read_d ;
 		} ;
 
-	if (psf->mode == SFM_WRITE)
+	if (psf->file.mode == SFM_WRITE)
 	{	psf->write_short	= dwvw_write_s ;
 		psf->write_int		= dwvw_write_i ;
 		psf->write_float	= dwvw_write_f ;
 		psf->write_double	= dwvw_write_d ;
 		} ;
 
-	psf->seek	= dwvw_seek ;
-	psf->close	= dwvw_close ;
+	psf->codec_close = dwvw_close ;
+	psf->seek = dwvw_seek ;
 
-	/* FIXME : This s bogus. */
+	/* FIXME : This is bogus. */
 	psf->sf.frames = SF_COUNT_MAX ;
 	psf->datalength = psf->sf.frames ;
-	/* EMXIF : This s bogus. */
+	/* EMXIF : This is bogus. */
 
 	return 0 ;
 } /* dwvw_init */
@@ -127,11 +133,11 @@ static int
 dwvw_close (SF_PRIVATE *psf)
 {	DWVW_PRIVATE *pdwvw ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
-	if (psf->mode == SFM_WRITE)
+	if (psf->file.mode == SFM_WRITE)
 	{	static int last_values [12] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } ;
 
 		/* Write 8 zero samples to fully flush output. */
@@ -148,17 +154,15 @@ dwvw_close (SF_PRIVATE *psf)
 } /* dwvw_close */
 
 static sf_count_t
-dwvw_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset)
+dwvw_seek	(SF_PRIVATE *psf, int UNUSED (mode), sf_count_t offset)
 {	DWVW_PRIVATE *pdwvw ;
 
-	mode = mode ;
-
-	if (! psf->fdata)
+	if (! psf->codec_data)
 	{	psf->error = SFE_INTERNAL ;
-		return ((sf_count_t) -1) ;
+		return PSF_SEEK_ERROR ;
 		} ;
 
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	if (offset == 0)
 	{	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
@@ -167,7 +171,7 @@ dwvw_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset)
 		} ;
 
 	psf->error = SFE_BAD_SEEK ;
-	return	((sf_count_t) -1) ;
+	return	PSF_SEEK_ERROR ;
 } /* dwvw_seek */
 
 
@@ -181,12 +185,12 @@ dwvw_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int		k, bufferlen, readcount = 0, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = SF_BUFFER_LEN / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	readcount = (len >= bufferlen) ? bufferlen : len ;
 		count = dwvw_decode_data (psf, pdwvw, iptr, readcount) ;
@@ -208,9 +212,9 @@ dwvw_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	int			readcount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	while (len > 0)
 	{	readcount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -235,14 +239,14 @@ dwvw_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float	normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x80000000) : 1.0 ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = SF_BUFFER_LEN / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	readcount = (len >= bufferlen) ? bufferlen : len ;
 		count = dwvw_decode_data (psf, pdwvw, iptr, readcount) ;
@@ -266,14 +270,14 @@ dwvw_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double 	normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x80000000) : 1.0 ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = SF_BUFFER_LEN / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	readcount = (len >= bufferlen) ? bufferlen : len ;
 		count = dwvw_decode_data (psf, pdwvw, iptr, readcount) ;
@@ -467,7 +471,7 @@ dump_bits (DWVW_PRIVATE *pdwvw)
 				} ;
 
 static int
-dwvw_encode_data (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, int *ptr, int len)
+dwvw_encode_data (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, const int *ptr, int len)
 {	int	count ;
 	int delta_width_modifier, delta, delta_negative, delta_width, extra_bit ;
 
@@ -542,18 +546,18 @@ dwvw_encode_data (SF_PRIVATE *psf, DWVW_PRIVATE *pdwvw, int *ptr, int len)
 } /* dwvw_encode_data */
 
 static sf_count_t
-dwvw_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
+dwvw_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 {	DWVW_PRIVATE *pdwvw ;
 	int		*iptr ;
 	int		k, bufferlen, writecount = 0, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = SF_BUFFER_LEN / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : len ;
 		for (k = 0 ; k < writecount ; k++)
@@ -570,14 +574,14 @@ dwvw_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 } /* dwvw_write_s */
 
 static sf_count_t
-dwvw_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
+dwvw_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 {	DWVW_PRIVATE *pdwvw ;
 	int			writecount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	while (len > 0)
 	{	writecount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -595,21 +599,21 @@ dwvw_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 } /* dwvw_write_i */
 
 static sf_count_t
-dwvw_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
+dwvw_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 {	DWVW_PRIVATE *pdwvw ;
 	int			*iptr ;
 	int			k, bufferlen, writecount = 0, count ;
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFFFFFF) : 1.0 ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = SF_BUFFER_LEN / sizeof (short) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : len ;
 		for (k = 0 ; k < writecount ; k++)
@@ -626,21 +630,21 @@ dwvw_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 } /* dwvw_write_f */
 
 static sf_count_t
-dwvw_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
+dwvw_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 {	DWVW_PRIVATE *pdwvw ;
 	int			*iptr ;
 	int			k, bufferlen, writecount = 0, count ;
 	sf_count_t	total = 0 ;
 	double 		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pdwvw = (DWVW_PRIVATE*) psf->fdata ;
+	pdwvw = (DWVW_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFFFFFF) : 1.0 ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = SF_BUFFER_LEN / sizeof (short) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : len ;
 		for (k = 0 ; k < writecount ; k++)
@@ -655,10 +659,4 @@ dwvw_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	return total ;
 } /* dwvw_write_d */
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 1ca09552-b01f-4d7f-9bcf-612f834fe41d
-*/
+
