@@ -73,26 +73,24 @@ static void psf_log_syserr (SF_PRIVATE *psf, int error) ;
 */
 
 static int psf_close_fd (int fd) ;
-static int psf_open_fd (const char * path, int mode) ;
+static int psf_open_fd (PSF_FILE * pfile) ;
 static sf_count_t psf_get_filelen_fd (int fd) ;
 
 int
-psf_fopen (SF_PRIVATE *psf, const char *pathname, int open_mode)
+psf_fopen (SF_PRIVATE *psf)
 {
 	psf->error = 0 ;
-	psf->filedes = psf_open_fd (pathname, open_mode) ;
+	psf->file.filedes = psf_open_fd (&psf->file) ;
 
-	if (psf->filedes == - SFE_BAD_OPEN_MODE)
+	if (psf->file.filedes == - SFE_BAD_OPEN_MODE)
 	{	psf->error = SFE_BAD_OPEN_MODE ;
-		psf->filedes = -1 ;
+		psf->file.filedes = -1 ;
 		return psf->error ;
 		} ;
 
-	if (psf->filedes == -1)
+	if (psf->file.filedes == -1)
 		psf_log_syserr (psf, errno) ;
 
-	psf->mode = open_mode ;
-
 	return psf->error ;
 } /* psf_fopen */
 
@@ -103,37 +101,37 @@ psf_fclose (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return 0 ;
 
-	if (psf->do_not_close_descriptor)
-	{	psf->filedes = -1 ;
+	if (psf->file.do_not_close_descriptor)
+	{	psf->file.filedes = -1 ;
 		return 0 ;
 		} ;
 
-	if ((retval = psf_close_fd (psf->filedes)) == -1)
+	if ((retval = psf_close_fd (psf->file.filedes)) == -1)
 		psf_log_syserr (psf, errno) ;
 
-	psf->filedes = -1 ;
+	psf->file.filedes = -1 ;
 
 	return retval ;
 } /* psf_fclose */
 
 int
-psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
+psf_open_rsrc (SF_PRIVATE *psf)
 {
-	if (psf->rsrcdes > 0)
+	if (psf->rsrc.filedes > 0)
 		return 0 ;
 
 	/* Test for MacOSX style resource fork on HPFS or HPFS+ filesystems. */
-	LSF_SNPRINTF (psf->rsrcpath, sizeof (psf->rsrcpath), "%s/rsrc", psf->filepath) ;
+	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s/rsrc", psf->file.path.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrcdes = psf_open_fd (psf->rsrcpath, open_mode)) >= 0)
-	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrcdes) ;
-		if (psf->rsrclength > 0 || (open_mode & SFM_WRITE))
+	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
+		if (psf->rsrclength > 0 || (psf->rsrc.mode & SFM_WRITE))
 			return SFE_NO_ERROR ;
-		psf_close_fd (psf->rsrcdes) ;
-		psf->rsrcdes = -1 ;
+		psf_close_fd (psf->rsrc.filedes) ;
+		psf->rsrc.filedes = -1 ;
 		} ;
 
-	if (psf->rsrcdes == - SFE_BAD_OPEN_MODE)
+	if (psf->rsrc.filedes == - SFE_BAD_OPEN_MODE)
 	{	psf->error = SFE_BAD_OPEN_MODE ;
 		return psf->error ;
 		} ;
@@ -142,10 +140,10 @@ psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
 	** Now try for a resource fork stored as a separate file in the same
 	** directory, but preceded with a dot underscore.
 	*/
-	LSF_SNPRINTF (psf->rsrcpath, sizeof (psf->rsrcpath), "%s._%s", psf->directory, psf->filename) ;
+	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s._%s", psf->file.dir.c, psf->file.name.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrcdes = psf_open_fd (psf->rsrcpath, open_mode)) >= 0)
-	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrcdes) ;
+	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
 		return SFE_NO_ERROR ;
 		} ;
 
@@ -153,18 +151,18 @@ psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
 	** Now try for a resource fork stored in a separate file in the
 	** .AppleDouble/ directory.
 	*/
-	LSF_SNPRINTF (psf->rsrcpath, sizeof (psf->rsrcpath), "%s.AppleDouble/%s", psf->directory, psf->filename) ;
+	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s.AppleDouble/%s", psf->file.dir.c, psf->file.name.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->rsrcdes = psf_open_fd (psf->rsrcpath, open_mode)) >= 0)
-	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrcdes) ;
+	if ((psf->rsrc.filedes = psf_open_fd (&psf->rsrc)) >= 0)
+	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrc.filedes) ;
 		return SFE_NO_ERROR ;
 		} ;
 
 	/* No resource file found. */
-	if (psf->rsrcdes == -1)
+	if (psf->rsrc.filedes == -1)
 		psf_log_syserr (psf, errno) ;
 
-	psf->rsrcdes = -1 ;
+	psf->rsrc.filedes = -1 ;
 
 	return psf->error ;
 } /* psf_open_rsrc */
@@ -176,7 +174,7 @@ psf_get_filelen (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return psf->vio.get_filelen (psf->vio_user_data) ;
 
-	filelen = psf_get_filelen_fd (psf->filedes) ;
+	filelen = psf_get_filelen_fd (psf->file.filedes) ;
 
 	if (filelen == -1)
 	{	psf_log_syserr (psf, errno) ;
@@ -188,7 +186,7 @@ psf_get_filelen (SF_PRIVATE *psf)
 		return (sf_count_t) -1 ;
 		} ;
 
-	switch (psf->mode)
+	switch (psf->file.mode)
 	{	case SFM_WRITE :
 			filelen = filelen - psf->fileoffset ;
 			break ;
@@ -217,27 +215,27 @@ psf_get_filelen (SF_PRIVATE *psf)
 int
 psf_close_rsrc (SF_PRIVATE *psf)
 {
-	if (psf->rsrcdes >= 0)
-		psf_close_fd (psf->rsrcdes) ;
-	psf->rsrcdes = -1 ;
+	if (psf->rsrc.filedes >= 0)
+		psf_close_fd (psf->rsrc.filedes) ;
+	psf->rsrc.filedes = -1 ;
 	return 0 ;
 } /* psf_close_rsrc */
 
 int
-psf_set_stdio (SF_PRIVATE *psf, int mode)
+psf_set_stdio (SF_PRIVATE *psf)
 {	int	error = 0 ;
 
-	switch (mode)
+	switch (psf->file.mode)
 	{	case SFM_RDWR :
 				error = SFE_OPEN_PIPE_RDWR ;
 				break ;
 
 		case SFM_READ :
-				psf->filedes = 0 ;
+				psf->file.filedes = 0 ;
 				break ;
 
 		case SFM_WRITE :
-				psf->filedes = 1 ;
+				psf->file.filedes = 1 ;
 				break ;
 
 		default :
@@ -251,29 +249,31 @@ psf_set_stdio (SF_PRIVATE *psf, int mode)
 
 void
 psf_set_file (SF_PRIVATE *psf, int fd)
-{	psf->filedes = fd ;
+{	psf->file.filedes = fd ;
 } /* psf_set_file */
 
 int
 psf_file_valid (SF_PRIVATE *psf)
-{	return (psf->filedes >= 0) ? SF_TRUE : SF_FALSE ;
+{	return (psf->file.filedes >= 0) ? SF_TRUE : SF_FALSE ;
 } /* psf_set_file */
 
 sf_count_t
 psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
-{	sf_count_t	new_position ;
+{	sf_count_t	current_pos, new_position ;
 
 	if (psf->virtual_io)
 		return psf->vio.seek (offset, whence, psf->vio_user_data) ;
 
+	current_pos = psf_ftell (psf) ;
+
 	switch (whence)
 	{	case SEEK_SET :
 				offset += psf->fileoffset ;
 				break ;
 
 		case SEEK_END :
-				if (psf->mode == SFM_WRITE)
-				{	new_position = lseek (psf->filedes, offset, whence) ;
+				if (psf->file.mode == SFM_WRITE)
+				{	new_position = lseek (psf->file.filedes, offset, whence) ;
 
 					if (new_position < 0)
 						psf_log_syserr (psf, errno) ;
@@ -286,15 +286,25 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 				** get the offset wrt the start of file.
 				*/
 				whence = SEEK_SET ;
-				offset = lseek (psf->filedes, 0, SEEK_END) + offset ;
+				offset = lseek (psf->file.filedes, 0, SEEK_END) + offset ;
+				break ;
+
+		case SEEK_CUR :
+				/* Translate a SEEK_CUR into a SEEK_SET. */
+				offset += current_pos ;
+				whence = SEEK_SET ;
 				break ;
 
 		default :
-				/* No need to do anything about SEEK_CUR. */
-				break ;
+				/* We really should not be here. */
+				psf_log_printf (psf, "psf_fseek : whence is %d *****.\n", whence) ;
+				return 0 ;
 		} ;
 
-	new_position = lseek (psf->filedes, offset, whence) ;
+	if (current_pos != offset)
+		new_position = lseek (psf->file.filedes, offset, whence) ;
+	else
+		new_position = offset ;
 
 	if (new_position < 0)
 		psf_log_syserr (psf, errno) ;
@@ -322,7 +332,7 @@ psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 	{	/* Break the read down to a sensible size. */
 		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : (ssize_t) items ;
 
-		count = read (psf->filedes, ((char*) ptr) + total, (size_t) count) ;
+		count = read (psf->file.filedes, ((char*) ptr) + total, (size_t) count) ;
 
 		if (count == -1)
 		{	if (errno == EINTR)
@@ -363,7 +373,7 @@ psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf
 	{	/* Break the writes down to a sensible size. */
 		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : items ;
 
-		count = write (psf->filedes, ((const char*) ptr) + total, count) ;
+		count = write (psf->file.filedes, ((const char*) ptr) + total, count) ;
 
 		if (count == -1)
 		{	if (errno == EINTR)
@@ -396,7 +406,7 @@ psf_ftell (SF_PRIVATE *psf)
 	if (psf->is_pipe)
 		return psf->pipeoffset ;
 
-	pos = lseek (psf->filedes, 0, SEEK_CUR) ;
+	pos = lseek (psf->file.filedes, 0, SEEK_CUR) ;
 
 	if (pos == ((sf_count_t) -1))
 	{	psf_log_syserr (psf, errno) ;
@@ -422,7 +432,7 @@ psf_fgets (char *buffer, sf_count_t bufsize, SF_PRIVATE *psf)
 	sf_count_t		count ;
 
 	while (k < bufsize - 1)
-	{	count = read (psf->filedes, &(buffer [k]), 1) ;
+	{	count = read (psf->file.filedes, &(buffer [k]), 1) ;
 
 		if (count == -1)
 		{	if (errno == EINTR)
@@ -448,7 +458,7 @@ psf_is_pipe (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return SF_FALSE ;
 
-	if (fstat (psf->filedes, &statbuf) == -1)
+	if (fstat (psf->file.filedes, &statbuf) == -1)
 	{	psf_log_syserr (psf, errno) ;
 		/* Default to maximum safety. */
 		return SF_TRUE ;
@@ -488,7 +498,7 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 	if ((sizeof (off_t) < sizeof (sf_count_t)) && len > 0x7FFFFFFF)
 		return -1 ;
 
-	retval = ftruncate (psf->filedes, len) ;
+	retval = ftruncate (psf->file.filedes, len) ;
 
 	if (retval == -1)
 		psf_log_syserr (psf, errno) ;
@@ -498,28 +508,28 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 
 void
 psf_init_files (SF_PRIVATE *psf)
-{	psf->filedes = -1 ;
-	psf->rsrcdes = -1 ;
-	psf->savedes = -1 ;
+{	psf->file.filedes = -1 ;
+	psf->rsrc.filedes = -1 ;
+	psf->file.savedes = -1 ;
 } /* psf_init_files */
 
 void
 psf_use_rsrc (SF_PRIVATE *psf, int on_off)
 {
 	if (on_off)
-	{	if (psf->filedes != psf->rsrcdes)
-		{	psf->savedes = psf->filedes ;
-			psf->filedes = psf->rsrcdes ;
+	{	if (psf->file.filedes != psf->rsrc.filedes)
+		{	psf->file.savedes = psf->file.filedes ;
+			psf->file.filedes = psf->rsrc.filedes ;
 			} ;
 		}
-	else if (psf->filedes == psf->rsrcdes)
-		psf->filedes = psf->savedes ;
+	else if (psf->file.filedes == psf->rsrc.filedes)
+		psf->file.filedes = psf->file.savedes ;
 
 	return ;
 } /* psf_use_rsrc */
 
 static int
-psf_open_fd (const char * pathname, int open_mode)
+psf_open_fd (PSF_FILE * pfile)
 {	int fd, oflag, mode ;
 
 	/*
@@ -533,7 +543,7 @@ psf_open_fd (const char * pathname, int open_mode)
 		exit (1) ;
 		} ;
 
-	switch (open_mode)
+	switch (pfile->mode)
 	{	case SFM_READ :
 				oflag = O_RDONLY | O_BINARY ;
 				mode = 0 ;
@@ -554,10 +564,10 @@ psf_open_fd (const char * pathname, int open_mode)
 				break ;
 		} ;
 
-	if (mode == 0)
-		fd = open (pathname, oflag) ;
+	if (pfile->mode == 0)
+		fd = open (pfile->path.c, oflag) ;
 	else
-		fd = open (pathname, oflag, mode) ;
+		fd = open (pfile->path.c, oflag, mode) ;
 
 	return fd ;
 } /* psf_open_fd */
@@ -568,7 +578,7 @@ psf_log_syserr (SF_PRIVATE *psf, int error)
 	/* Only log an error if no error has been set yet. */
 	if (psf->error == 0)
 	{	psf->error = SFE_SYSTEM ;
-		LSF_SNPRINTF (psf->syserr, sizeof (psf->syserr), "System error : %s.", strerror (error)) ;
+		snprintf (psf->syserr, sizeof (psf->syserr), "System error : %s.", strerror (error)) ;
 		} ;
 
 	return ;
@@ -578,8 +588,8 @@ void
 psf_fsync (SF_PRIVATE *psf)
 {
 #if HAVE_FSYNC
-    if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-        fsync (psf->filedes) ;
+    if (psf->file.mode == SFM_WRITE || psf->file.mode == SFM_RDWR)
+        fsync (psf->file.filedes) ;
 #else
     psf = NULL ;
 #endif
@@ -593,20 +603,18 @@ psf_fsync (SF_PRIVATE *psf)
 #include <io.h>
 
 static int psf_close_handle (HANDLE handle) ;
-static HANDLE psf_open_handle (const char * path, int mode) ;
+static HANDLE psf_open_handle (PSF_FILE * pfile) ;
 static sf_count_t psf_get_filelen_handle (HANDLE handle) ;
 
 /* USE_WINDOWS_API */ int
-psf_fopen (SF_PRIVATE *psf, const char *pathname, int open_mode)
+psf_fopen (SF_PRIVATE *psf)
 {
 	psf->error = 0 ;
-	psf->hfile = psf_open_handle (pathname, open_mode) ;
+	psf->file.handle = psf_open_handle (&psf->file) ;
 
-	if (psf->hfile == NULL)
+	if (psf->file.handle == NULL)
 		psf_log_syserr (psf, GetLastError ()) ;
 
-	psf->mode = open_mode ;
-
 	return psf->error ;
 } /* psf_fopen */
 
@@ -617,30 +625,30 @@ psf_fclose (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return 0 ;
 
-	if (psf->do_not_close_descriptor)
-	{	psf->hfile = NULL ;
+	if (psf->file.do_not_close_descriptor)
+	{	psf->file.handle = NULL ;
 		return 0 ;
 		} ;
 
-	if ((retval = psf_close_handle (psf->hfile)) == -1)
+	if ((retval = psf_close_handle (psf->file.handle)) == -1)
 		psf_log_syserr (psf, GetLastError ()) ;
 
-	psf->hfile = NULL ;
+	psf->file.handle = NULL ;
 
 	return retval ;
 } /* psf_fclose */
 
 /* USE_WINDOWS_API */ int
-psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
+psf_open_rsrc (SF_PRIVATE *psf)
 {
-	if (psf->hrsrc != NULL)
+	if (psf->rsrc.handle != NULL)
 		return 0 ;
 
 	/* Test for MacOSX style resource fork on HPFS or HPFS+ filesystems. */
-	LSF_SNPRINTF (psf->rsrcpath, sizeof (psf->rsrcpath), "%s/rsrc", psf->filepath) ;
+	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s/rsrc", psf->file.path.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->hrsrc = psf_open_handle (psf->rsrcpath, open_mode)) != NULL)
-	{	psf->rsrclength = psf_get_filelen_handle (psf->hrsrc) ;
+	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
+	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
 		return SFE_NO_ERROR ;
 		} ;
 
@@ -648,10 +656,10 @@ psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
 	** Now try for a resource fork stored as a separate file in the same
 	** directory, but preceded with a dot underscore.
 	*/
-	LSF_SNPRINTF (psf->rsrcpath, sizeof (psf->rsrcpath), "%s._%s", psf->directory, psf->filename) ;
+	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s._%s", psf->file.dir.c, psf->file.name.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->hrsrc = psf_open_handle (psf->rsrcpath, open_mode)) != NULL)
-	{	psf->rsrclength = psf_get_filelen_handle (psf->hrsrc) ;
+	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
+	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
 		return SFE_NO_ERROR ;
 		} ;
 
@@ -659,18 +667,18 @@ psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
 	** Now try for a resource fork stored in a separate file in the
 	** .AppleDouble/ directory.
 	*/
-	LSF_SNPRINTF (psf->rsrcpath, sizeof (psf->rsrcpath), "%s.AppleDouble/%s", psf->directory, psf->filename) ;
+	snprintf (psf->rsrc.path.c, sizeof (psf->rsrc.path.c), "%s.AppleDouble/%s", psf->file.dir.c, psf->file.name.c) ;
 	psf->error = SFE_NO_ERROR ;
-	if ((psf->hrsrc = psf_open_handle (psf->rsrcpath, open_mode)) != NULL)
-	{	psf->rsrclength = psf_get_filelen_handle (psf->hrsrc) ;
-		return SFE_NO_ERROR ;
+	if ((psf->rsrc.handle = psf_open_handle (&psf->rsrc)) != NULL)
+	{	psf->rsrclength = psf_get_filelen_handle (psf->rsrc.handle) ;
+      		return SFE_NO_ERROR ;
 		} ;
 
 	/* No resource file found. */
-	if (psf->hrsrc == NULL)
+	if (psf->rsrc.handle == NULL)
 		psf_log_syserr (psf, GetLastError ()) ;
 
-	psf->hrsrc = NULL ;
+	psf->rsrc.handle = NULL ;
 
 	return psf->error ;
 } /* psf_open_rsrc */
@@ -682,7 +690,7 @@ psf_get_filelen (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return psf->vio.get_filelen (psf->vio_user_data) ;
 
-	filelen = psf_get_filelen_handle (psf->hfile) ;
+	filelen = psf_get_filelen_handle (psf->file.handle) ;
 
 	if (filelen == -1)
 	{	psf_log_syserr (psf, errno) ;
@@ -694,7 +702,7 @@ psf_get_filelen (SF_PRIVATE *psf)
 		return (sf_count_t) -1 ;
 		} ;
 
-	switch (psf->mode)
+	switch (psf->file.mode)
 	{	case SFM_WRITE :
 			filelen = filelen - psf->fileoffset ;
 			break ;
@@ -722,34 +730,34 @@ psf_get_filelen (SF_PRIVATE *psf)
 
 /* USE_WINDOWS_API */ void
 psf_init_files (SF_PRIVATE *psf)
-{	psf->hfile = NULL ;
-	psf->hrsrc = NULL ;
-	psf->hsaved = NULL ;
+{	psf->file.handle = NULL ;
+	psf->rsrc.handle = NULL ;
+	psf->file.hsaved = NULL ;
 } /* psf_init_files */
 
 /* USE_WINDOWS_API */ void
 psf_use_rsrc (SF_PRIVATE *psf, int on_off)
 {
 	if (on_off)
-	{	if (psf->hfile != psf->hrsrc)
-		{	psf->hsaved = psf->hfile ;
-			psf->hfile = psf->hrsrc ;
+	{	if (psf->file.handle != psf->rsrc.handle)
+		{	psf->file.hsaved = psf->file.handle ;
+			psf->file.handle = psf->rsrc.handle ;
 			} ;
 		}
-	else if (psf->hfile == psf->hrsrc)
-		psf->hfile = psf->hsaved ;
+	else if (psf->file.handle == psf->rsrc.handle)
+		psf->file.handle = psf->file.hsaved ;
 
 	return ;
 } /* psf_use_rsrc */
 
 /* USE_WINDOWS_API */ static HANDLE
-psf_open_handle (const char * pathname, int open_mode)
+psf_open_handle (PSF_FILE * pfile)
 {	DWORD dwDesiredAccess ;
 	DWORD dwShareMode ;
 	DWORD dwCreationDistribution ;
 	HANDLE handle ;
 
-	switch (open_mode)
+	switch (pfile->mode)
 	{	case SFM_READ :
 				dwDesiredAccess = GENERIC_READ ;
 				dwShareMode = FILE_SHARE_READ | FILE_SHARE_WRITE ;
@@ -772,15 +780,26 @@ psf_open_handle (const char * pathname, int open_mode)
 				return NULL ;
 		} ;
 
-	handle = CreateFile (
-			pathname,					/* pointer to name of the file */
-			dwDesiredAccess,			/* access (read-write) mode */
-			dwShareMode,				/* share mode */
-			0,							/* pointer to security attributes */
-			dwCreationDistribution,		/* how to create */
-			FILE_ATTRIBUTE_NORMAL,		/* file attributes (could use FILE_FLAG_SEQUENTIAL_SCAN) */
-			NULL						/* handle to file with attributes to copy */
-			) ;
+	if (pfile->use_wchar)
+		handle = CreateFileW (
+					pfile->path.wc,				/* pointer to name of the file */
+					dwDesiredAccess,			/* access (read-write) mode */
+					dwShareMode,				/* share mode */
+					0,							/* pointer to security attributes */
+					dwCreationDistribution,		/* how to create */
+					FILE_ATTRIBUTE_NORMAL,		/* file attributes (could use FILE_FLAG_SEQUENTIAL_SCAN) */
+					NULL						/* handle to file with attributes to copy */
+					) ;
+	else
+		handle = CreateFile (
+					pfile->path.c,				/* pointer to name of the file */
+					dwDesiredAccess,			/* access (read-write) mode */
+					dwShareMode,				/* share mode */
+					0,							/* pointer to security attributes */
+					dwCreationDistribution,		/* how to create */
+					FILE_ATTRIBUTE_NORMAL,		/* file attributes (could use FILE_FLAG_SEQUENTIAL_SCAN) */
+					NULL						/* handle to file with attributes to copy */
+					) ;
 
 	if (handle == INVALID_HANDLE_VALUE)
 		return NULL ;
@@ -806,7 +825,7 @@ psf_log_syserr (SF_PRIVATE *psf, int error)
 			NULL
 			) ;
 
-		LSF_SNPRINTF (psf->syserr, sizeof (psf->syserr), "System error : %s", lpMsgBuf) ;
+		snprintf (psf->syserr, sizeof (psf->syserr), "System error : %s", (char*) lpMsgBuf) ;
 		LocalFree (lpMsgBuf) ;
 		} ;
 
@@ -817,31 +836,31 @@ psf_log_syserr (SF_PRIVATE *psf, int error)
 /* USE_WINDOWS_API */ int
 psf_close_rsrc (SF_PRIVATE *psf)
 {
-	if (psf->hrsrc != NULL)
-		psf_close_handle (psf->hrsrc) ;
-	psf->hrsrc = NULL ;
+	if (psf->rsrc.handle != NULL)
+		psf_close_handle (psf->rsrc.handle) ;
+	psf->rsrc.handle = NULL ;
 	return 0 ;
 } /* psf_close_rsrc */
 
 
 /* USE_WINDOWS_API */ int
-psf_set_stdio (SF_PRIVATE *psf, int mode)
+psf_set_stdio (SF_PRIVATE *psf)
 {	HANDLE	handle = NULL ;
 	int	error = 0 ;
 
-	switch (mode)
+	switch (psf->file.mode)
 	{	case SFM_RDWR :
 				error = SFE_OPEN_PIPE_RDWR ;
 				break ;
 
 		case SFM_READ :
 				handle = GetStdHandle (STD_INPUT_HANDLE) ;
-				psf->do_not_close_descriptor = 1 ;
+				psf->file.do_not_close_descriptor = 1 ;
 				break ;
 
 		case SFM_WRITE :
 				handle = GetStdHandle (STD_OUTPUT_HANDLE) ;
-				psf->do_not_close_descriptor = 1 ;
+				psf->file.do_not_close_descriptor = 1 ;
 				break ;
 
 		default :
@@ -849,7 +868,7 @@ psf_set_stdio (SF_PRIVATE *psf, int mode)
 				break ;
 		} ;
 
-	psf->hfile = handle ;
+	psf->file.handle = handle ;
 	psf->filelength = 0 ;
 
 	return error ;
@@ -863,14 +882,14 @@ psf_set_file (SF_PRIVATE *psf, int fd)
 	osfhandle = _get_osfhandle (fd) ;
 	handle = (HANDLE) osfhandle ;
 
-	psf->hfile = handle ;
+	psf->file.handle = handle ;
 } /* psf_set_file */
 
 /* USE_WINDOWS_API */ int
 psf_file_valid (SF_PRIVATE *psf)
-{	if (psf->hfile == NULL)
+{	if (psf->file.handle == NULL)
 		return SF_FALSE ;
-	if (psf->hfile == INVALID_HANDLE_VALUE)
+	if (psf->file.handle == INVALID_HANDLE_VALUE)
 		return SF_FALSE ;
 	return SF_TRUE ;
 } /* psf_set_file */
@@ -903,7 +922,7 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 	lDistanceToMove = (DWORD) (offset & 0xFFFFFFFF) ;
 	lDistanceToMoveHigh = (DWORD) ((offset >> 32) & 0xFFFFFFFF) ;
 
-	dwResult = SetFilePointer (psf->hfile, lDistanceToMove, &lDistanceToMoveHigh, dwMoveMethod) ;
+	dwResult = SetFilePointer (psf->file.handle, lDistanceToMove, &lDistanceToMoveHigh, dwMoveMethod) ;
 
 	if (dwResult == 0xFFFFFFFF)
 		dwError = GetLastError () ;
@@ -939,7 +958,7 @@ psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 	{	/* Break the writes down to a sensible size. */
 		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : (ssize_t) items ;
 
-		if (ReadFile (psf->hfile, ((char*) ptr) + total, count, &dwNumberOfBytesRead, 0) == 0)
+		if (ReadFile (psf->file.handle, ((char*) ptr) + total, count, &dwNumberOfBytesRead, 0) == 0)
 		{	psf_log_syserr (psf, GetLastError ()) ;
 			break ;
 			}
@@ -978,7 +997,7 @@ psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf
 	{	/* Break the writes down to a sensible size. */
 		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : (ssize_t) items ;
 
-		if (WriteFile (psf->hfile, ((const char*) ptr) + total, count, &dwNumberOfBytesWritten, 0) == 0)
+		if (WriteFile (psf->file.handle, ((const char*) ptr) + total, count, &dwNumberOfBytesWritten, 0) == 0)
 		{	psf_log_syserr (psf, GetLastError ()) ;
 			break ;
 			}
@@ -1013,7 +1032,7 @@ psf_ftell (SF_PRIVATE *psf)
 	lDistanceToMoveLow = 0 ;
 	lDistanceToMoveHigh = 0 ;
 
-	dwResult = SetFilePointer (psf->hfile, lDistanceToMoveLow, &lDistanceToMoveHigh, FILE_CURRENT) ;
+	dwResult = SetFilePointer (psf->file.handle, lDistanceToMoveLow, &lDistanceToMoveHigh, FILE_CURRENT) ;
 
 	if (dwResult == 0xFFFFFFFF)
 		dwError = GetLastError () ;
@@ -1045,7 +1064,7 @@ psf_fgets (char *buffer, sf_count_t bufsize, SF_PRIVATE *psf)
 	DWORD dwNumberOfBytesRead ;
 
 	while (k < bufsize - 1)
-	{	if (ReadFile (psf->hfile, &(buffer [k]), 1, &dwNumberOfBytesRead, 0) == 0)
+	{	if (ReadFile (psf->file.handle, &(buffer [k]), 1, &dwNumberOfBytesRead, 0) == 0)
 		{	psf_log_syserr (psf, GetLastError ()) ;
 			break ;
 			}
@@ -1068,7 +1087,7 @@ psf_is_pipe (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return SF_FALSE ;
 
-	if (GetFileType (psf->hfile) == FILE_TYPE_DISK)
+	if (GetFileType (psf->file.handle) == FILE_TYPE_DISK)
 		return SF_FALSE ;
 
 	/* Default to maximum safety. */
@@ -1095,7 +1114,7 @@ psf_get_filelen_handle (HANDLE handle)
 
 /* USE_WINDOWS_API */ void
 psf_fsync (SF_PRIVATE *psf)
-{	FlushFileBuffers (psf->hfile) ;
+{	FlushFileBuffers (psf->file.handle) ;
 } /* psf_fsync */
 
 
@@ -1117,7 +1136,7 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 	lDistanceToMoveLow = (DWORD) (len & 0xFFFFFFFF) ;
 	lDistanceToMoveHigh = (DWORD) ((len >> 32) & 0xFFFFFFFF) ;
 
-	dwResult = SetFilePointer (psf->hfile, lDistanceToMoveLow, &lDistanceToMoveHigh, FILE_BEGIN) ;
+	dwResult = SetFilePointer (psf->file.handle, lDistanceToMoveLow, &lDistanceToMoveHigh, FILE_BEGIN) ;
 
 	if (dwResult == 0xFFFFFFFF)
 		dwError = GetLastError () ;
@@ -1132,7 +1151,7 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 		** which guarantees that the new portion of the file will be zeroed.
 		** Not sure if this is important or not.
 		*/
-		if (SetEndOfFile (psf->hfile) == 0)
+		if (SetEndOfFile (psf->file.handle) == 0)
 		{	retval = -1 ;
 			psf_log_syserr (psf, GetLastError ()) ;
 			} ;
@@ -1188,14 +1207,14 @@ psf_fopen (SF_PRIVATE *psf, const char *pathname, int open_mode)
 		} ;
 
 	if (mode == 0)
-		psf->filedes = open (pathname, oflag) ;
+		psf->file.filedes = open (pathname, oflag) ;
 	else
-		psf->filedes = open (pathname, oflag, mode) ;
+		psf->file.filedes = open (pathname, oflag, mode) ;
 
-	if (psf->filedes == -1)
+	if (psf->file.filedes == -1)
 		psf_log_syserr (psf, errno) ;
 
-	return psf->filedes ;
+	return psf->file.filedes ;
 } /* psf_fopen */
 
 /* Win32 */ sf_count_t
@@ -1211,8 +1230,8 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 				break ;
 
 		case SEEK_END :
-				if (psf->mode == SFM_WRITE)
-				{	new_position = _lseeki64 (psf->filedes, offset, whence) ;
+				if (psf->file.mode == SFM_WRITE)
+				{	new_position = _lseeki64 (psf->file.filedes, offset, whence) ;
 
 					if (new_position < 0)
 						psf_log_syserr (psf, errno) ;
@@ -1225,7 +1244,7 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 				** get the offset wrt the start of file.
 				*/
 				whence = SEEK_SET ;
-				offset = _lseeki64 (psf->filedes, 0, SEEK_END) + offset ;
+				offset = _lseeki64 (psf->file.filedes, 0, SEEK_END) + offset ;
 				break ;
 
 		default :
@@ -1240,9 +1259,9 @@ psf_fseek (SF_PRIVATE *psf, sf_count_t offset, int whence)
 	*** Use the _telli64() function instead.
 	*/
 	if (offset == 0 && whence == SEEK_CUR)
-		new_position = _telli64 (psf->filedes) ;
+		new_position = _telli64 (psf->file.filedes) ;
 	else
-		new_position = _lseeki64 (psf->filedes, offset, whence) ;
+		new_position = _lseeki64 (psf->file.filedes, offset, whence) ;
 
 	if (new_position < 0)
 		psf_log_syserr (psf, errno) ;
@@ -1270,7 +1289,7 @@ psf_fread (void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf)
 	{	/* Break the writes down to a sensible size. */
 		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : (ssize_t) items ;
 
-		count = read (psf->filedes, ((char*) ptr) + total, (size_t) count) ;
+		count = read (psf->file.filedes, ((char*) ptr) + total, (size_t) count) ;
 
 		if (count == -1)
 		{	if (errno == EINTR)
@@ -1308,7 +1327,7 @@ psf_fwrite (const void *ptr, sf_count_t bytes, sf_count_t items, SF_PRIVATE *psf
 	{	/* Break the writes down to a sensible size. */
 		count = (items > SENSIBLE_SIZE) ? SENSIBLE_SIZE : items ;
 
-		count = write (psf->filedes, ((const char*) ptr) + total, count) ;
+		count = write (psf->file.filedes, ((const char*) ptr) + total, count) ;
 
 		if (count == -1)
 		{	if (errno == EINTR)
@@ -1335,7 +1354,7 @@ psf_ftell (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return psf->vio.tell (psf->vio_user_data) ;
 
-	pos = _telli64 (psf->filedes) ;
+	pos = _telli64 (psf->file.filedes) ;
 
 	if (pos == ((sf_count_t) -1))
 	{	psf_log_syserr (psf, errno) ;
@@ -1349,13 +1368,13 @@ psf_ftell (SF_PRIVATE *psf)
 psf_fclose (SF_PRIVATE *psf)
 {	int retval ;
 
-	while ((retval = close (psf->filedes)) == -1 && errno == EINTR)
+	while ((retval = close (psf->file.filedes)) == -1 && errno == EINTR)
 		/* Do nothing. */ ;
 
 	if (retval == -1)
 		psf_log_syserr (psf, errno) ;
 
-	psf->filedes = -1 ;
+	psf->file.filedes = -1 ;
 
 	return retval ;
 } /* psf_fclose */
@@ -1366,7 +1385,7 @@ psf_fgets (char *buffer, sf_count_t bufsize, SF_PRIVATE *psf)
 	sf_count_t	count ;
 
 	while (k < bufsize - 1)
-	{	count = read (psf->filedes, &(buffer [k]), 1) ;
+	{	count = read (psf->file.filedes, &(buffer [k]), 1) ;
 
 		if (count == -1)
 		{	if (errno == EINTR)
@@ -1393,7 +1412,7 @@ psf_is_pipe (SF_PRIVATE *psf)
 		return SF_FALSE ;
 
 	/* Not sure if this works. */
-	if (fstat (psf->filedes, &statbuf) == -1)
+	if (fstat (psf->file.filedes, &statbuf) == -1)
 	{	psf_log_syserr (psf, errno) ;
 		/* Default to maximum safety. */
 		return SF_TRUE ;
@@ -1417,7 +1436,7 @@ psf_get_filelen (SF_PRIVATE *psf)
 	*/
 	struct _stati64 statbuf ;
 
-	if (_fstati64 (psf->filedes, &statbuf))
+	if (_fstati64 (psf->file.filedes, &statbuf))
 	{	psf_log_syserr (psf, errno) ;
 		return (sf_count_t) -1 ;
 		} ;
@@ -1429,7 +1448,7 @@ psf_get_filelen (SF_PRIVATE *psf)
 	if (psf->virtual_io)
 		return psf->vio.get_filelen (psf->vio_user_data) ;
 
-	if ((current = _telli64 (psf->filedes)) < 0)
+	if ((current = _telli64 (psf->file.filedes)) < 0)
 	{	psf_log_syserr (psf, errno) ;
 		return (sf_count_t) -1 ;
 		} ;
@@ -1447,17 +1466,17 @@ psf_get_filelen (SF_PRIVATE *psf)
 	** by libsndfile and changing the license to GPL at the same time.
 	*/
 
-	_lseeki64 (psf->filedes, 0, SEEK_END) ;
+	_lseeki64 (psf->file.filedes, 0, SEEK_END) ;
 
-	if ((filelen = _lseeki64 (psf->filedes, 0, SEEK_END)) < 0)
+	if ((filelen = _lseeki64 (psf->file.filedes, 0, SEEK_END)) < 0)
 	{	psf_log_syserr (psf, errno) ;
 		return (sf_count_t) -1 ;
 		} ;
 
 	if (filelen > current)
-		_lseeki64 (psf->filedes, current, SEEK_SET) ;
+		_lseeki64 (psf->file.filedes, current, SEEK_SET) ;
 
-	switch (psf->mode)
+	switch (psf->file.mode)
 	{	case SFM_WRITE :
 			filelen = filelen - psf->fileoffset ;
 			break ;
@@ -1501,7 +1520,7 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 	if (len > 0x7FFFFFFF)
 		return -1 ;
 
-	retval = chsize (psf->filedes, len) ;
+	retval = chsize (psf->file.filedes, len) ;
 
 	if (retval == -1)
 		psf_log_syserr (psf, errno) ;
@@ -1516,7 +1535,7 @@ psf_log_syserr (SF_PRIVATE *psf, int error)
 	/* Only log an error if no error has been set yet. */
 	if (psf->error == 0)
 	{	psf->error = SFE_SYSTEM ;
-		LSF_SNPRINTF (psf->syserr, sizeof (psf->syserr), "System error : %s", strerror (error)) ;
+		snprintf (psf->syserr, sizeof (psf->syserr), "System error : %s", strerror (error)) ;
 		} ;
 
 	return ;
