@@ -120,10 +120,6 @@ static FLAC__StreamEncoderSeekStatus sf_flac_enc_seek_callback (const FLAC__Stre
 static FLAC__StreamEncoderTellStatus sf_flac_enc_tell_callback (const FLAC__StreamEncoder *encoder, FLAC__uint64 *absolute_byte_offset, void *client_data) ;
 static FLAC__StreamEncoderWriteStatus sf_flac_enc_write_callback (const FLAC__StreamEncoder *encoder, const FLAC__byte buffer [], size_t bytes, unsigned samples, unsigned current_frame, void *client_data) ;
 
-static const int legal_sample_rates [] =
-{	8000, 16000, 22050, 24000, 32000, 44100, 48000, 96000
-} ;
-
 static void
 s2flac8_array (const short *src, FLAC__int32 *dest, int count)
 {	while (--count >= 0)
@@ -618,17 +614,17 @@ flac_open	(SF_PRIVATE *psf)
 	FLAC_PRIVATE* pflac = calloc (1, sizeof (FLAC_PRIVATE)) ;
 	psf->codec_data = pflac ;
 
-	if (psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_RDWR)
 		return SFE_BAD_MODE_RW ;
 
-	if (psf->mode == SFM_READ)
+	if (psf->file.mode == SFM_READ)
 	{	if ((error = flac_read_header (psf)))
 			return error ;
 		} ;
 
 	subformat = SF_CODEC (psf->sf.format) ;
 
-	if (psf->mode == SFM_WRITE)
+	if (psf->file.mode == SFM_WRITE)
 	{	if ((SF_CONTAINER (psf->sf.format)) != SF_FORMAT_FLAC)
 			return	SFE_BAD_OPEN_FORMAT ;
 
@@ -681,7 +677,7 @@ flac_close	(SF_PRIVATE *psf)
 	if (pflac->metadata != NULL)
 		FLAC__metadata_object_delete (pflac->metadata) ;
 
-	if (psf->mode == SFM_WRITE)
+	if (psf->file.mode == SFM_WRITE)
 	{	FLAC__stream_encoder_finish (pflac->fse) ;
 		FLAC__stream_encoder_delete (pflac->fse) ;
 
@@ -689,7 +685,7 @@ flac_close	(SF_PRIVATE *psf)
 			free (pflac->encbuffer) ;
 		} ;
 
-	if (psf->mode == SFM_READ)
+	if (psf->file.mode == SFM_READ)
 	{	FLAC__stream_decoder_finish (pflac->fsd) ;
 		FLAC__stream_decoder_delete (pflac->fsd) ;
 		} ;
@@ -707,17 +703,16 @@ static int
 flac_enc_init (SF_PRIVATE *psf)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
 	unsigned bps ;
-	int k, found ;
 
-	found = 0 ;
-	for (k = 0 ; k < ARRAY_LEN (legal_sample_rates) ; k++)
-		if (psf->sf.samplerate == legal_sample_rates [k])
-		{	found = 1 ;
-			break ;
-			} ;
-
-	if (found == 0)
+	/* To cite the flac FAQ at
+	** http://flac.sourceforge.net/faq.html#general__samples
+	**     "FLAC supports linear sample rates from 1Hz - 655350Hz in 1Hz
+	**     increments."
+	*/
+	if ( psf->sf.samplerate < 1 || psf->sf.samplerate > 655350 )
+	{	psf_log_printf (psf, "flac sample rate out of range.\n", psf->sf.samplerate) ;
 		return SFE_FLAC_BAD_SAMPLE_RATE ;
+		} ;
 
 	psf_fseek (psf, 0, SEEK_SET) ;
 
@@ -794,17 +789,17 @@ flac_command (SF_PRIVATE * UNUSED (psf), int UNUSED (command), void * UNUSED (da
 int
 flac_init (SF_PRIVATE *psf)
 {
-	if (psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_RDWR)
 		return SFE_BAD_MODE_RW ;
 
-	if (psf->mode == SFM_READ)
+	if (psf->file.mode == SFM_READ)
 	{	psf->read_short		= flac_read_flac2s ;
 		psf->read_int		= flac_read_flac2i ;
 		psf->read_float		= flac_read_flac2f ;
 		psf->read_double	= flac_read_flac2d ;
 		} ;
 
-	if (psf->mode == SFM_WRITE)
+	if (psf->file.mode == SFM_WRITE)
 	{	psf->write_short	= flac_write_s2flac ;
 		psf->write_int		= flac_write_i2flac ;
 		psf->write_float	= flac_write_f2flac ;
@@ -1286,7 +1281,7 @@ flac_seek (SF_PRIVATE *psf, int UNUSED (mode), sf_count_t offset)
 
 	pflac->frame = NULL ;
 
-	if (psf->mode == SFM_READ)
+	if (psf->file.mode == SFM_READ)
 	{	FLAC__uint64 position ;
 		if (FLAC__stream_decoder_seek_absolute (pflac->fsd, offset))
 		{	FLAC__stream_decoder_get_decode_position (pflac->fsd, &position) ;
