@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -53,7 +53,7 @@ int
 interleave_init	(SF_PRIVATE *psf)
 {	INTERLEAVE_DATA *pdata ;
 
-	if (psf->mode != SFM_READ)
+	if (psf->file.mode != SFM_READ)
 		return SFE_INTERLEAVE_MODE ;
 
 	if (psf->interleave)
@@ -297,10 +297,4 @@ interleave_seek	(SF_PRIVATE *psf, int mode, sf_count_t samples_from_start)
 
 	return samples_from_start ;
 } /* interleave_seek */
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 82314e13-0225-4408-a2f2-e6dab3f38904
-*/
+
