@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,13 +16,14 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"sfconfig.h"
+
 #include	<stdio.h>
 #include	<string.h>
 #include	<ctype.h>
 #include	<stdarg.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
 #include	"common.h"
 
@@ -32,7 +33,7 @@ int
 rx2_open	(SF_PRIVATE *psf)
 {	if (psf)
 		return SFE_UNIMPLEMENTED ;
-	return (psf && 0) ;
+	return 0 ;
 } /* rx2_open */
 
 #else
@@ -74,7 +75,7 @@ static int	rx2_close	(SF_PRIVATE *psf) ;
 
 int
 rx2_open	(SF_PRIVATE *psf)
-{	static char *marker_type [4] =
+{	static const char *marker_type [4] =
 	{	"Original Enabled", "Enabled Hidden",
 		"Additional/PencilTool", "Disabled"
 		} ;
@@ -118,14 +119,14 @@ rx2_open	(SF_PRIVATE *psf)
 	/* Get name length */
 	length = 0 ;
 	psf_binheader_readf (psf, "1", &length) ;
-	if (length >= SIGNED_SIZEOF (psf->buffer))
+	if (length >= SIGNED_SIZEOF (psf->u.cbuf))
 	{	psf_log_printf (psf, "  Text : %d *** Error : Too sf_count_t!\n") ;
 		return -1001 ;
 		}
 
-	memset (psf->buffer, 0, SIGNED_SIZEOF (psf->buffer)) ;
-	psf_binheader_readf (psf, "b", psf->buffer, length) ;
-	psf_log_printf (psf, " Text : \"%s\"\n", psf->buffer) ;
+	memset (psf->u.cbuf, 0, sizeof (psf->u.cbuf)) ;
+	psf_binheader_readf (psf, "b", psf->u.cbuf, length) ;
+	psf_log_printf (psf, " Text : \"%s\"\n", psf->u.cbuf) ;
 
 	/* Jump to GLOB offset position. */
 	if (glob_offset & 1)
@@ -289,7 +290,7 @@ rx2_open	(SF_PRIVATE *psf)
 	if ((error = dwvw_init (psf, 16)))
 		return error ;
 
-	psf->close = rx2_close ;
+	psf->container_close = rx2_close ;
 
 	if (! psf->sf.frames && psf->blockwidth)
 		psf->sf.frames = psf->datalength / psf->blockwidth ;
@@ -305,7 +306,7 @@ rx2_open	(SF_PRIVATE *psf)
 static int
 rx2_close	(SF_PRIVATE *psf)
 {
-	if (psf->mode == SFM_WRITE)
+	if (psf->file.mode == SFM_WRITE)
 	{	/*  Now we know for certain the length of the file we can re-write
 		**	correct values for the FORM, 8SVX and BODY chunks.
 		*/
@@ -316,10 +317,3 @@ rx2_close	(SF_PRIVATE *psf)
 } /* rx2_close */
 
 #endif
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 7366e813-9fee-4d1f-881e-e4a691469370
-*/
