@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,17 +16,22 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#ifndef SFENDIAN_INCLUDED
+#define SFENDIAN_INCLUDED
+
+#include "sfconfig.h"
 
 #if HAVE_STDINT_H
 #include <stdint.h>
+#elif HAVE_INTTYPES_H
+#include <inttypes.h>
 #endif
 
 #if (defined (SIZEOF_INT64_T) && (SIZEOF_INT64_T == 8))
 /* Good, we have int64_t. */
-#elif (defined  (SIZEOF_LONG_LONG) && (SIZEOF_LONG_LONG == 8))
+#elif (defined (SIZEOF_LONG_LONG) && (SIZEOF_LONG_LONG == 8))
 typedef long long int64_t ;
-#elif (defined  (SIZEOF_LONG) && (SIZEOF_LONG == 8))
+#elif (defined (SIZEOF_LONG) && (SIZEOF_LONG == 8))
 typedef long int64_t ;
 #elif (defined (WIN32) || defined (_WIN32))
 typedef __int64 int64_t ;
@@ -34,8 +39,6 @@ typedef __int64 int64_t ;
 #error "No 64 bit integer type."
 #endif
 
-#undef HAVE_BYTESWAP_H
-
 #if HAVE_BYTESWAP_H
 
 #include <byteswap.h>
@@ -45,9 +48,9 @@ typedef __int64 int64_t ;
 
 #else
 
-#define	ENDSWAP_SHORT(x)	((((x)>>8)&0xFF)+(((x)&0xFF)<<8))
-#define	ENDSWAP_INT(x)		((((x)>>24)&0xFF)+(((x)>>8)&0xFF00)+(((x)&0xFF00)<<8)+(((x)&0xFF)<<24))
-	
+#define	ENDSWAP_SHORT(x)	((((x) >> 8) & 0xFF) + (((x) & 0xFF) << 8))
+#define	ENDSWAP_INT(x)		((((x) >> 24) & 0xFF) + (((x) >> 8) & 0xFF00) + (((x) & 0xFF00) << 8) + (((x) & 0xFF) << 24))
+
 #endif
 
 /*
@@ -58,11 +61,11 @@ typedef __int64 int64_t ;
 */
 
 #if (CPU_IS_LITTLE_ENDIAN == 1)
-	#define	MAKE_MARKER(a,b,c,d)	((a)|((b)<<8)|((c)<<16)|((d)<<24))
+	#define	MAKE_MARKER(a,b,c,d)	((a) | ((b) << 8) | ((c) << 16) | ((d) << 24))
 #elif (CPU_IS_BIG_ENDIAN == 1)
-	#define	MAKE_MARKER(a,b,c,d)	(((a)<<24)|((b)<<16)|((c)<<8)|(d))
+	#define	MAKE_MARKER(a,b,c,d)	(((a) << 24) | ((b) << 16) | ((c) << 8) | (d))
 #else
-	#error "Target CPU endian-ness unknown. May need to hand edit src/config.h"
+	#error "Target CPU endian-ness unknown. May need to hand edit src/sfconfig.h"
 #endif
 
 /*
@@ -75,27 +78,25 @@ typedef __int64 int64_t ;
 #if (CPU_IS_LITTLE_ENDIAN == 1)
 	#define LES2H_SHORT(x)			(x)
 	#define LEI2H_INT(x)			(x)
-	#define BES2H_SHORT(x)			ENDSWAP_SHORT(x)
-	#define BEI2H_INT(x)			ENDSWAP_INT(x)
 
-	#define H2BE_SHORT(x)			ENDSWAP_SHORT(x)
-	#define H2BE_INT(x)				ENDSWAP_INT(x)
-	#define H2LE_SHORT(x)			(x)
-	#define H2LE_INT(x)				(x)
+	#define BES2H_SHORT(x)			ENDSWAP_SHORT (x)
+	#define BEI2H_INT(x)			ENDSWAP_INT (x)
+
+	#define H2BE_SHORT(x)			ENDSWAP_SHORT (x)
+	#define H2BE_INT(x)				ENDSWAP_INT (x)
 
 #elif (CPU_IS_BIG_ENDIAN == 1)
-	#define LES2H_SHORT(x)			ENDSWAP_SHORT(x)
-	#define LEI2H_INT(x)			ENDSWAP_INT(x)
+	#define LES2H_SHORT(x)			ENDSWAP_SHORT (x)
+	#define LEI2H_INT(x)			ENDSWAP_INT (x)
+
 	#define BES2H_SHORT(x)			(x)
 	#define BEI2H_INT(x)			(x)
 
-	#define H2BE_SHORT(x)			(x)
-	#define H2BE_INT(x)				(x)
-	#define H2LE_SHORT(x)			ENDSWAP_SHORT(x)
-	#define H2LE_INT(x)				ENDSWAP_INT(x)
+	#define H2LE_SHORT(x)			ENDSWAP_SHORT (x)
+	#define H2LE_INT(x)				ENDSWAP_INT (x)
 
 #else
-	#error "Target CPU endian-ness unknown. May need to hand edit src/config.h"
+	#error "Target CPU endian-ness unknown. May need to hand edit src/sfconfig.h"
 #endif
 
 #define LET2H_SHORT_PTR(x)		((x) [1] + ((x) [2] << 8))
@@ -202,7 +203,7 @@ static inline void
 endswap_int64_t_copy (int64_t *dest, const int64_t *src, int len)
 {	const unsigned char *psrc ;
 	unsigned char *pdest ;
-	
+
 	if (dest == src)
 	{	endswap_int64_t_array (dest, len) ;
 		return ;
@@ -249,10 +250,5 @@ endswap_double_copy (double *dest, const double *src, int len)
 {	endswap_int64_t_copy ((int64_t *) dest, (const int64_t *) src, len) ;
 } /* endswap_double_copy */
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch
-** revision control system.
-**
-** arch-tag: f0c5cd54-42d3-4237-90ec-11fe24995de7
-*/
+#endif /* SFENDIAN_INCLUDED */
+
