@@ -107,14 +107,14 @@ voc_open	(SF_PRIVATE *psf)
 	if (psf->is_pipe)
 		return SFE_VOC_NO_PIPE ;
 
-	if (psf->mode == SFM_READ || (psf->mode == SFM_RDWR && psf->filelength > 0))
+	if (psf->file.mode == SFM_READ || (psf->file.mode == SFM_RDWR && psf->filelength > 0))
 	{	if ((error = voc_read_header (psf)))
 			return error ;
 		} ;
 
 	subformat = SF_CODEC (psf->sf.format) ;
 
-	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_WRITE || psf->file.mode == SFM_RDWR)
 	{	if ((SF_CONTAINER (psf->sf.format)) != SF_FORMAT_VOC)
 			return	SFE_BAD_OPEN_FORMAT ;
 
@@ -528,11 +528,11 @@ voc_write_header (SF_PRIVATE *psf, int calc_length)
 static int
 voc_close	(SF_PRIVATE *psf)
 {
-	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
+	if (psf->file.mode == SFM_WRITE || psf->file.mode == SFM_RDWR)
 	{	/*  Now we know for certain the length of the file we can re-write
 		**	correct values for the FORM, 8SVX and BODY chunks.
 		*/
-		unsigned byte = VOC_TERMINATOR ;
+		unsigned char byte = VOC_TERMINATOR ;
 
 
 		psf_fseek (psf, 0, SEEK_END) ;
