@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -16,14 +16,14 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_UNISTD_H
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
@@ -138,7 +138,7 @@ stdin_test	(int typemajor, int count)
 
 	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
-	int			k, total ;
+	int			k, total, err ;
 
 	if (typemajor == SF_FORMAT_RAW)
 	{	sfinfo.samplerate	= 44100 ;
@@ -147,15 +147,21 @@ stdin_test	(int typemajor, int count)
 		sfinfo.frames		= 0 ;
 		}
 	else
-		sfinfo.format = 0 ;
+		memset (&sfinfo, 0, sizeof (sfinfo)) ;
 
-	if (! (file = sf_open ("-", SFM_READ, &sfinfo)))
-	{	fprintf (stderr, "sf_open_read failed with error : ") ;
+	if ((file = sf_open_fd (STDIN_FILENO, SFM_READ, &sfinfo, SF_TRUE)) == NULL)
+	{	fprintf (stderr, "sf_open_fd failed with error : ") ;
 		puts (sf_strerror (NULL)) ;
 		dump_log_buffer (NULL) ;
 		exit (1) ;
 		} ;
 
+	err = sf_error (file) ;
+	if (err != SF_ERR_NO_ERROR)
+	{	printf ("Line %d : unexpected error : %s\n", __LINE__, sf_error_number (err)) ;
+		exit (1) ;
+		} ;
+
 	if ((sfinfo.format & SF_FORMAT_TYPEMASK) != typemajor)
 	{	fprintf (stderr, "\n\nError : File type doesn't match.\n") ;
 		exit (1) ;
@@ -195,10 +201,4 @@ stdin_test	(int typemajor, int count)
 
 	return ;
 } /* stdin_test */
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 5e470e3e-743e-4bbc-a588-ec883843c938
-*/
+
