@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@ main (void)
 	const char	*filename ;
 	int			k ;
 
+	print_test_name ("ulaw_test", "encoder") ;
+
 	filename = "test.raw" ;
 
 	sfinfo.format		= SF_FORMAT_RAW | SF_FORMAT_ULAW ;
@@ -94,7 +96,9 @@ main (void)
 
 	sf_close (file) ;
 
-	printf ("    ulaw_test : encoder ... ok\n") ;
+	puts ("ok") ;
+
+	print_test_name ("ulaw_test", "decoder") ;
 
 	/* Now generate a file containing all possible 8 bit encoded
 	** sample values and write it to disk as ulaw encoded.frames.
@@ -139,7 +143,7 @@ main (void)
 
 	sf_close (file) ;
 
-	printf ("    ulaw_test : decoder ... ok\n") ;
+	puts ("ok") ;
 
 	unlink (filename) ;
 
@@ -253,10 +257,3 @@ int ulaw_decode (unsigned int ulawbyte)
     return sample ;
 } /* ulaw_decode */
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: e764cee1-5a9a-480b-a4ca-34d9b57dea6f
-*/
