@@ -32,7 +32,6 @@
 
 #include <string.h>
 #include <fcntl.h>
-#include <errno.h>
 #include <sys/types.h>
 
 #include "utils.h"
@@ -48,7 +47,7 @@
 
 
 static const char * locations [] =
-{	"../src/", "src/", "../src/.libs/", "src/.libs/",
+{	".", "../src/", "src/", "../src/.libs/", "src/.libs/",
 	NULL
 } ; /* locations. */
 
